@@ -2,11 +2,11 @@
 **  Copyright (c) 2007, 2008 Sendmail, Inc. and its suppliers.
 **	All rights reserved.
 **
-**  $Id: test.c,v 1.34 2008/06/05 15:12:44 msk Exp $
+**  $Id: test.c,v 1.35 2008/10/31 03:06:04 msk Exp $
 */
 
 #ifndef lint
-static char test_c_id[] = "@(#)$Id: test.c,v 1.34 2008/06/05 15:12:44 msk Exp $";
+static char test_c_id[] = "@(#)$Id: test.c,v 1.35 2008/10/31 03:06:04 msk Exp $";
 #endif /* !lint */
 
 /* system includes */
@@ -639,10 +639,42 @@ dkimf_testfile(DKIM_LIB *libdkim, char *file, time_t fixedtime, bool strict,
 
 				if (selector != NULL || domain != NULL)
 				{
+#ifdef USE_UNBOUND
+					char *dnssec;
+					int dnsseccode = DKIM_DNSSEC_UNKNOWN;
+				
+					dnsseccode = dkim_sig_getdnssec(sig);
+
+					switch (dnsseccode)
+					{
+					  case DKIM_DNSSEC_BOGUS:
+						dnssec = "bogus";
+						break;
+
+					  case DKIM_DNSSEC_INSECURE:
+						dnssec = "insecure";
+						break;
+
+					  case DKIM_DNSSEC_SECURE:
+						dnssec = "secure";
+						break;
+
+					  case DKIM_DNSSEC_UNKNOWN:
+					  default:
+						dnssec = "unknown";
+						break;
+					}
+
+					fprintf(stdout,
+					        "%s: %s: verification (s=%s d=%s, %d-bit key, %s) failed: %s\n",
+					        progname, file, selector,
+					        domain, keysize, dnssec, err);
+#else /* USE_UNBOUND */
 					fprintf(stdout,
 					        "%s: %s: verification (s=%s d=%s, %d-bit key) failed: %s\n",
 					        progname, file, selector,
 					        domain, keysize, err);
+#endif /* USE_UNBOUND */
 				}
 				else
 				{
