@@ -2,11 +2,11 @@
 **  Copyright (c) 2005-2008 Sendmail, Inc. and its suppliers.
 **	All rights reserved.
 **
-**  $Id: util.c,v 1.74.4.1 2008/08/19 19:46:13 msk Exp $
+**  $Id: util.c,v 1.83 2008/12/15 19:59:38 msk Exp $
 */
 
 #ifndef lint
-static char util_c_id[] = "@(#)$Id: util.c,v 1.74.4.1 2008/08/19 19:46:13 msk Exp $";
+static char util_c_id[] = "@(#)$Id: util.c,v 1.83 2008/12/15 19:59:38 msk Exp $";
 #endif /* !lint */
 
 /* system includes */
@@ -74,14 +74,14 @@ static char *optlist[] =
 	"QUERY_CACHE",
 #endif /* QUERY_CACHE */
 
+#if USE_UNBOUND
+	"USE_UNBOUND",
+#endif /* USE_UNBOUND */
+
 #if VERIFY_DOMAINKEYS
 	"VERIFY_DOMAINKEYS",
 #endif /* VERIFY_DOMAINKEYS */
 
-#if _FFR_ANTICIPATE_SENDMAIL_MUNGE
-	"_FFR_ANTICIPATE_SENDMAIL_MUNGE",
-#endif /* _FFR_ANTICIPATE_SENDMAIL_MUNGE */
-
 #ifdef _FFR_BODYLENGTH_DB
 	"_FFR_BODYLENGTH_DB",
 #endif /* _FFR_BODYLENGTH_DB */
@@ -90,14 +90,26 @@ static char *optlist[] =
 	"_FFR_CAPTURE_UNKNOWN_ERRORS",
 #endif /* _FFR_CAPTURE_UNKNOWN_ERRORS */
 
+#if _FFR_COMMAIZE
+	"_FFR_COMMAIZE",
+#endif /* _FFR_COMMAIZE */
+
 #if _FFR_DIFFHEADERS
 	"_FFR_DIFFHEADERS",
 #endif /* _FFR_DIFFHEADERS */
 
+#if _FFR_DKIM_REPUTATION
+	"_FFR_DKIM_REPUTATION",
+#endif /* _FFR_DKIM_REPUTATION */
+
 #if _FFR_DNS_UPGRADE
 	"_FFR_DNS_UPGRADE",
 #endif /* _FFR_DNS_UPGRADE */
 
+#if _FFR_MULTIPLE_SIGNATURES
+	"_FFR_MULTIPLE_SIGNATURES",
+#endif /* _FFR_MULTIPLE_SIGNATURES */
+
 #if _FFR_PARSE_TIME
 	"_FFR_PARSE_TIME",
 #endif /* _FFR_PARSE_TIME */
@@ -106,10 +118,6 @@ static char *optlist[] =
 	"_FFR_REPLACE_RULES",
 #endif /* _FFR_REPLACE_RULES */
 
-#if _FFR_REQUIRED_HEADERS
-	"_FFR_REQUIRED_HEADERS",
-#endif /* _FFR_REQUIRED_HEADERS */
-
 #if _FFR_SELECT_CANONICALIZATION
 	"_FFR_SELECT_CANONICALIZATION",
 #endif /* _FFR_SELECT_CANONICALIZATION */
@@ -562,7 +570,7 @@ dkimf_checkpopauth(DB *db, struct sockaddr *ip)
 
 	dkimf_inet_ntoa(addr, ipbuf, sizeof ipbuf);
 	exists = FALSE;
-	status = dkimf_db_exists(db, ipbuf, &exists, &pop_lock);
+	status = dkimf_db_get(db, ipbuf, NULL, NULL, &exists, &pop_lock);
 	return (status == 0 && exists);
 }
 #endif /* POPAUTH */
@@ -1568,3 +1576,178 @@ dkimf_base64_encode_file(infd, out, lm, rm, initial)
 		fputc('=', out);
 	}
 }
+
+#ifdef _FFR_COMMAIZE
+/*
+**  DKIMF_COMMAIZE -- reformat a header field which contains a comma-separated
+**                    list of addresses; attempts to mimic what
+**                    commaize() in sendmail/headers.c does
+**
+**  Parameters:
+**  	dst -- destination string object
+**  	src -- source string
+**  	ilen -- initial string length
+**  	margin -- wrap length
+** 	noleadspc -- "no leading spaces" has been set (i.e. milter v2 in use)
+**
+**  Return value:
+**  	Number of addresses found. 
+*/
+
+int
+dkimf_commaize(struct dkimf_dstring *dst, char *src, size_t ilen,
+               size_t margin, bool noleadspc)
+{
+	bool incomment;
+	bool infocus;
+	bool quoted;
+	bool escaped;
+	bool first;
+	int acnt;
+	size_t len;
+	size_t alen;
+	char *p;
+	char *end;
+	char *start = NULL;
+
+	assert(dst != NULL);
+	assert(src != NULL);
+
+	end = src + strlen(src);
+	len = ilen;
+	incomment = FALSE;
+	infocus = FALSE;
+	quoted = FALSE;
+	escaped = FALSE;
+	first = TRUE;
+	acnt = 0;
+
+	for (p = src; p < end; p++)
+	{
+		if (isascii(*p) && isspace(*p))
+		{
+			/* don't eat the first space unless appropriate */
+			if (p == src && noleadspc)
+				dkimf_dstring_cat1(dst, ' ');
+			continue;
+		}
+
+		if (*p == '(' && !escaped)
+		{
+			incomment = TRUE;
+			if (start == NULL)
+				start = p;
+			continue;
+		}
+
+		if (*p == ')' && !escaped)
+		{
+			incomment = FALSE;
+			if (start == NULL)
+				start = p;
+			continue;
+		}
+
+		if (*p == '<' && !escaped)
+		{
+			infocus = TRUE;
+			if (start == NULL)
+				start = p;
+			continue;
+		}
+
+		if (*p == '>' && !escaped)
+		{
+			infocus = FALSE;
+			if (start == NULL)
+				start = p;
+			continue;
+		}
+
+		if (*p == '\"' && !escaped)
+		{
+			quoted = !quoted;
+			if (start == NULL)
+				start = p;
+			continue;
+		}
+
+		if (escaped)
+		{
+			escaped = FALSE;
+			if (start == NULL)
+				start = p;
+			continue;
+		}
+
+		if (*p == '\\')
+		{
+			escaped = TRUE;
+			if (start == NULL)
+				start = p;
+			continue;
+		}
+
+		if (*p == ',' && !incomment && !infocus && !quoted)
+		{
+			if (start == NULL)
+				continue;
+
+			*p = '\0';
+
+			alen = strlen(start);
+			if (len + alen + 2 > margin && !first)
+			{
+				if (acnt > 0)
+				{
+					dkimf_dstring_cat(dst,
+					                  ",\r\n        ");
+				}
+				dkimf_dstring_cat(dst, start);
+				len = 8 + alen;
+				first = TRUE;
+			}
+			else
+			{
+				if (acnt > 0)
+					dkimf_dstring_cat(dst, ", ");
+				dkimf_dstring_cat(dst, start);
+				len = len + alen + 2;
+				first = FALSE;
+			}
+
+			*p = ',';
+
+			start = NULL;
+
+			acnt++;
+
+			continue;
+		}
+
+		if (start == NULL)
+			start = p;
+	}
+
+	if (start != NULL)
+	{
+		alen = strlen(start);
+
+		if (len + alen + 2 > margin && !first)
+		{
+			dkimf_dstring_cat(dst, ",\r\n        ");
+			dkimf_dstring_cat(dst, start);
+		}
+		else
+		{
+			if (acnt > 0)
+				dkimf_dstring_cat(dst, ", ");
+			dkimf_dstring_cat(dst, start);
+		}
+
+		acnt++;
+	}
+
+	return acnt;
+}
+#endif /* _FFR_COMMAIZE */
