@@ -6,7 +6,7 @@
 #ifdef QUERY_CACHE
 
 #ifndef lint
-static char dkim_cache_c_id[] = "@(#)$Id: dkim-cache.c,v 1.26 2008/04/24 20:21:05 msk Exp $";
+static char dkim_cache_c_id[] = "@(#)$Id: dkim-cache.c,v 1.27 2008/07/22 20:42:38 msk Exp $";
 #endif /* !lint */
 
 /* system includes */
@@ -20,6 +20,9 @@ static char dkim_cache_c_id[] = "@(#)$Id: dkim-cache.c,v 1.26 2008/04/24 20:21:0
 #include <pthread.h>
 
 /* libsm includes */
+#include <sm/gen.h>
+#include <sm/types.h>
+#include <sm/cdefs.h>
 #include <sm/string.h>
 
 /* libdb includes */
