@@ -4,7 +4,7 @@
 */
 
 #ifndef lint
-static char dkim_canon_c_id[] = "@(#)$Id: dkim-canon.c,v 1.49 2008/07/22 20:42:38 msk Exp $";
+static char dkim_canon_c_id[] = "@(#)$Id: dkim-canon.c,v 1.51 2008/11/26 01:20:28 msk Exp $";
 #endif /* !lint */
 
 /* system includes */
@@ -929,7 +929,9 @@ dkim_add_canon(DKIM *dkim, bool hdr, dkim_canon_t canon, int hashtype,
 **  	A DKIM_STAT_* constant.
 **
 **  Note:
-**  	Header canonicalizations are finalized by this function.
+**  	Header canonicalizations are finalized by this function when
+**  	verifying.  In signing mode, header canonicalizations are finalized
+**  	by a subsequent call to dkim_canon_signature().
 */
 
 DKIM_STAT
@@ -989,7 +991,7 @@ dkim_canon_runheaders(DKIM *dkim, bool signing)
 				for (hdr = dkim->dkim_hhead;
 				     hdr != NULL;
 				     hdr = hdr->hdr_next)
-					hdr->hdr_flags |= ~DKIM_HDR_SIGNED;
+					hdr->hdr_flags |= DKIM_HDR_SIGNED;
 			}
 			else
 			{
@@ -1241,6 +1243,9 @@ dkim_canon_runheaders(DKIM *dkim, bool signing)
 **
 **  Return value:
 **  	A DKIM_STAT_* constant.
+**
+**  Notes:
+**  	Header canonicalizations are finalized by this function.
 */
 
 DKIM_STAT
