@@ -4,7 +4,7 @@
 */
 
 #ifndef lint
-static char dkim_keys_c_id[] = "@(#)$Id: dkim-keys.c,v 1.39.4.1 2008/08/21 01:43:40 msk Exp $";
+static char dkim_keys_c_id[] = "@(#)$Id: dkim-keys.c,v 1.43 2008/12/05 22:03:02 msk Exp $";
 #endif /* !lint */
 
 /* system includes */
@@ -37,6 +37,9 @@ static char dkim_keys_c_id[] = "@(#)$Id: dkim-keys.c,v 1.39.4.1 2008/08/21 01:43
 #include "dkim-keys.h"
 #include "dkim-cache.h"
 #include "dkim-test.h"
+#if USE_UNBOUND
+# include "dkim-ub.h"
+#endif /* USE_UNBOUND */
 #include "util.h"
 #include "dkim.h"
 
@@ -82,6 +85,9 @@ dkim_get_key_dns(DKIM *dkim, DKIM_SIGINFO *sig, u_char *buf, size_t buflen)
 	int type = -1;
 	int class = -1;
 	size_t anslen;
+#if USE_UNBOUND
+	struct dkim_unbound_cb_data cb_data;
+#endif /* USE_UNBOUND */
 #if USE_ARLIB
 	AR_LIB ar;
 	AR_QUERY q;
@@ -133,8 +139,41 @@ dkim_get_key_dns(DKIM *dkim, DKIM_SIGINFO *sig, u_char *buf, size_t buflen)
 	anslen = dkim_test_dns_get(dkim, ansbuf, sizeof ansbuf);
 	if (anslen == -1)
 	{
-#if USE_ARLIB
-# ifdef _FFR_DNS_UPGRADE
+#if USE_UNBOUND
+		status = dkim_unbound_queue(dkim, qname, T_TXT,
+		                            buf, buflen, &cb_data);
+
+		if (status != 0)
+		{
+			dkim_error(dkim, "error queueing DNS request for `%s'",
+			           qname);
+			return DKIM_STAT_KEYFAIL;
+		}
+
+		status = dkim_unbound_wait(dkim, &cb_data);
+
+		switch (status)
+		{
+		  case 1:
+			sig->sig_dnssec_key = cb_data.ubd_result;
+			break;
+
+		  case 0:
+			dkim_error(dkim, "timeout DNS query for `%s'", qname);
+			return DKIM_STAT_KEYFAIL;
+
+		  case -1:
+			dkim_error(dkim, "error processing DNS query for `%s'",
+			           qname);
+			if (cb_data.ubd_stat != DKIM_STAT_OK)
+				return cb_data.ubd_stat;
+			else
+				return DKIM_STAT_INTERNAL;
+		}
+	}
+#else /* USE_UNBOUND */
+# if USE_ARLIB
+#  ifdef _FFR_DNS_UPGRADE
 		for (c = 0; c < 2; c++)
 		{
 			switch (c)
@@ -197,7 +236,7 @@ dkim_get_key_dns(DKIM *dkim, DKIM_SIGINFO *sig, u_char *buf, size_t buflen)
 
 			break;
 		}
-# else /* _FFR_DNS_UPGRADE */
+#  else /* _FFR_DNS_UPGRADE */
 		ar = dkim->dkim_libhandle->dkiml_arlib;
 
 		timeout.tv_sec = dkim->dkim_timeout;
@@ -235,12 +274,12 @@ dkim_get_key_dns(DKIM *dkim, DKIM_SIGINFO *sig, u_char *buf, size_t buflen)
 		}
 
 		(void) ar_cancelquery(ar, q);
-# endif /* _FFR_DNS_UPGRADE */
-#else /* USE_ARLIB */
+#  endif /* _FFR_DNS_UPGRADE */
+# else /* USE_ARLIB */
 		status = res_query(qname, C_IN, T_TXT, ansbuf, sizeof ansbuf);
-#endif /* USE_ARLIB */
+# endif /* USE_ARLIB */
 
-#if USE_ARLIB
+# if USE_ARLIB
 		if (status == AR_STAT_ERROR || status == AR_STAT_EXPIRED)
 		{
 			dkim_error(dkim, "ar_waitreply(): `%s' %s", qname,
@@ -248,7 +287,7 @@ dkim_get_key_dns(DKIM *dkim, DKIM_SIGINFO *sig, u_char *buf, size_t buflen)
 			                                   : "expired");
 			return DKIM_STAT_KEYFAIL;
 		}
-#else /* USE_ARLIB */
+# else /* USE_ARLIB */
 		/*
 		**  A -1 return from res_query could mean a bunch of things,
 		**  not just NXDOMAIN.  You can use h_errno to determine what
@@ -273,7 +312,7 @@ dkim_get_key_dns(DKIM *dkim, DKIM_SIGINFO *sig, u_char *buf, size_t buflen)
 		}
 
 		anslen = status;
-#endif /* USE_ARLIB */
+# endif /* USE_ARLIB */
 	}
 
 	/* set up pointers */
@@ -355,13 +394,13 @@ dkim_get_key_dns(DKIM *dkim, DKIM_SIGINFO *sig, u_char *buf, size_t buflen)
 		GETSHORT(type, cp);
 		GETSHORT(class, cp);
 
-#ifdef QUERY_CACHE
+# ifdef QUERY_CACHE
 		/* get the TTL */
 		GETLONG(ttl, cp);
-#else /* QUERY_CACHE */
+# else /* QUERY_CACHE */
 		/* skip the TTL */
 		cp += INT32SZ;
-#endif /* QUERY_CACHE */
+# endif /* QUERY_CACHE */
 
 		/* skip CNAME if found; assume it was resolved */
 		if (type == T_CNAME)
@@ -432,6 +471,7 @@ dkim_get_key_dns(DKIM *dkim, DKIM_SIGINFO *sig, u_char *buf, size_t buflen)
 			n--;
 		}
 	}
+#endif /* USE_UNBOUND */
 
 #ifdef QUERY_CACHE
 	if (!cached && buf[0] != '\0' &&
