@@ -4,7 +4,7 @@
 */
 
 #ifndef lint
-static char dkim_policy_c_id[] = "@(#)$Id: dkim-policy.c,v 1.41.4.2 2008/08/19 20:11:31 msk Exp $";
+static char dkim_policy_c_id[] = "@(#)$Id: dkim-policy.c,v 1.44 2008/10/31 21:26:49 msk Exp $";
 #endif /* !lint */
 
 /* system includes */
@@ -39,6 +39,9 @@ static char dkim_policy_c_id[] = "@(#)$Id: dkim-policy.c,v 1.41.4.2 2008/08/19 2
 #ifdef QUERY_CACHE
 # include "dkim-cache.h"
 #endif /* QUERY_CACHE */
+#ifdef USE_UNBOUND
+# include "dkim-ub.h"
+#endif /* USE_UNBOUND */
 #include "dkim-test.h"
 #include "util.h"
 
@@ -168,11 +171,20 @@ dkim_get_policy_file(DKIM *dkim, unsigned char *query, unsigned char *buf,
 int
 dkim_get_policy_dns_excheck(DKIM *dkim, unsigned char *query, int *qstatus)
 {
+#if USE_ARLIB
 	int c;
 	size_t anslen_a;
 	size_t anslen_aaaa;
 	size_t anslen_mx;
+#endif /* USE_ARLIB */
 	int status;
+#if USE_UNBOUND
+	int c;
+	int qtype;
+	unsigned char *outbuf;
+	size_t outsize;
+	struct dkim_unbound_cb_data cb_data;
+#endif /* USE_UNBOUND */
 	DKIM_LIB *lib;
 	HEADER hdr;
 #if USE_ARLIB
@@ -195,6 +207,79 @@ dkim_get_policy_dns_excheck(DKIM *dkim, unsigned char *query, int *qstatus)
 
 	lib = dkim->dkim_libhandle;
 
+#if USE_UNBOUND
+	for (c = 0; c < 3; c++)
+	{
+		if (c == 0)
+		{
+			qtype = T_A;
+			outbuf = ansbuf_a;
+			outsize = sizeof ansbuf_a;
+		}
+# ifdef T_AAAA
+		else if (c == 1)
+		{
+			qtype = T_AAAA;			/* AAAA */
+			outbuf = ansbuf_aaaa;
+			outsize = sizeof ansbuf_aaaa;
+		}
+		else if (c == 2)
+		{
+			qtype = T_MX;			/* MX */
+			outbuf = ansbuf_mx;
+			outsize = sizeof ansbuf_mx;
+		}
+# else /* T_AAAA */
+		else if (c == 1)
+		{
+			qtype = T_MX;			/* MX */
+			outbuf = ansbuf_mx;
+			outsize = sizeof ansbuf_mx;
+		}
+		else if (c == 2)
+		{
+			break;
+		}
+# endif /* T_AAAA */
+
+		/* query */
+		status = dkim_unbound_queue(dkim, query, qtype, 
+		                            outbuf, outsize, &cb_data);
+
+		if (status != 0)
+		{
+			dkim_error(dkim, "error queueing DNS request for `%s'",
+			           query);
+			return -1;
+		}
+
+		status = dkim_unbound_wait(dkim, &cb_data);
+
+		switch (status)
+		{
+		  case 1:
+			break;
+
+		  case 0:
+			dkim_error(dkim, "timeout DNS query for `%s'", query);
+			return -1;
+
+		  case -1:
+			dkim_error(dkim,
+			           "error processing DNS query for `%s'",
+			           query);
+			return -1;
+		}
+
+		*qstatus = cb_data.ubd_rcode;
+
+		if (cb_data.ubd_rcode == NOERROR)
+			return 1;
+	}
+
+	return 0;
+#endif /* USE_UNBOUND */
+
 #if USE_ARLIB
 # ifdef _FFR_DNS_UPGRADE
 	for (c = 0; c < 2; c++)
@@ -443,6 +528,7 @@ dkim_get_policy_dns_excheck(DKIM *dkim, unsigned char *query, int *qstatus)
 	return 0;
 }
 
+
 /*
 **  DKIM_GET_POLICY_DNS -- acquire a domain's policy record using DNS queries
 **
@@ -474,6 +560,10 @@ dkim_get_policy_dns(DKIM *dkim, unsigned char *query, bool excheck,
 #ifdef QUERY_CACHE
 	int ttl;
 #endif /* QUERY_CACHE */
+#if USE_UNBOUND
+	struct dkim_unbound_cb_data cb_data;
+	struct dkim_unbound_cb_data *cb_data_ptr = &cb_data;
+#endif /* USE_UNBOUND */
 	size_t anslen;
 #if USE_ARLIB
 	AR_LIB ar;
@@ -528,7 +618,48 @@ dkim_get_policy_dns(DKIM *dkim, unsigned char *query, bool excheck,
 			return dkim_get_policy_dns_excheck(dkim, query,
 			                                   qstatus);
 		}
+#ifdef USE_UNBOUND
+	status = dkim_unbound_queue(dkim, query, T_TXT, buf, buflen, &cb_data);
 
+	if (status != 0)
+	{
+		dkim_error(dkim, "error queueing DNS request for `%s'",
+		           query);
+		return -1;
+	}
+
+	status = dkim_unbound_wait(dkim, &cb_data);
+
+	switch (status)
+	{
+	  case 1:
+		dkim->dkim_dnssec_policy = cb_data_ptr->ubd_result;
+		break;
+
+	  case 0:
+		dkim_error(dkim, "timeout DNS query for `%s'", query);
+		return -1;
+
+	  case -1:
+		dkim_error(dkim, "error processing DNS query for `%s'", query);
+		return -1;
+	}
+
+	if (cb_data_ptr->ubd_stat != DKIM_STAT_OK)
+	{
+		if (cb_data_ptr->ubd_stat == DKIM_STAT_NOKEY)
+		{
+			return 0;
+		}
+		else
+		{
+			dkim_error(dkim, "error processing DNS policy query");
+			return -1;
+		}
+	}
+}
+
+#else /* USE_UNBOUND */
 #if USE_ARLIB
 # ifdef _FFR_DNS_UPGRADE
 		for (c = 0; c < 2; c++)
@@ -853,6 +984,7 @@ dkim_get_policy_dns(DKIM *dkim, unsigned char *query, bool excheck,
 #endif /* QUERY_CACHE */
 
 	sm_strlcpy(buf, outbuf, buflen);
+#endif /* USE_UNBOUND */
 
 	return 1;
 }
