@@ -4,7 +4,7 @@
 */
 
 #ifndef lint
-static char dkim_test_c_id[] = "@(#)$Id: dkim-test.c,v 1.11 2008/05/27 21:53:33 msk Exp $";
+static char dkim_test_c_id[] = "@(#)$Id: dkim-test.c,v 1.15 2008/07/22 20:42:38 msk Exp $";
 #endif /* !lint */
 
 /* system includes */
@@ -21,9 +21,9 @@ static char dkim_test_c_id[] = "@(#)$Id: dkim-test.c,v 1.11 2008/05/27 21:53:33
 
 /* libsm includes */
 #include <sm/gen.h>
-#include <sm/string.h>
 #include <sm/types.h>
 #include <sm/cdefs.h>
+#include <sm/string.h>
 
 /* openssl includes */
 #include <openssl/bio.h>
@@ -458,7 +458,8 @@ dkim_test_key(DKIM_LIB *lib, char *selector, char *domain,
 }
 
 /*
-**  DKIM_TEST_ASP -- verify that a valid author signing policy exists in dns
+**  DKIM_TEST_ADSP -- verify that a valid author domain signing policy exists
+**                    in DNS
 **
 **  Parameters:
 **  	lib -- DKIM library handle
@@ -474,8 +475,8 @@ dkim_test_key(DKIM_LIB *lib, char *selector, char *domain,
 */
 
 int
-dkim_test_asp(DKIM_LIB *lib, char *domain, dkim_policy_t *presult,
-              int *presult2, char *err, size_t errlen)
+dkim_test_adsp(DKIM_LIB *lib, const char *domain, dkim_policy_t *presult,
+               int *presult2, char *err, size_t errlen)
 {
 	DKIM_STAT stat;
 	dkim_policy_t pcode = DKIM_POLICY_DEFAULT;
@@ -494,7 +495,7 @@ dkim_test_asp(DKIM_LIB *lib, char *domain, dkim_policy_t *presult,
 	}
 
 	dkim->dkim_mode = DKIM_MODE_VERIFY;
-	dkim->dkim_domain = domain;
+	dkim->dkim_domain = (char *) domain;
 	dkim->dkim_sigcount = 0;
 
 	stat = dkim_policy(dkim, &pcode, NULL);
