@@ -4,7 +4,7 @@
 */
 
 #ifndef lint
-static char dkim_util_c_id[] = "@(#)$Id: dkim-util.c,v 1.23 2008/07/22 20:42:38 msk Exp $";
+static char dkim_util_c_id[] = "@(#)$Id: dkim-util.c,v 1.24 2008/09/23 15:34:43 msk Exp $";
 #endif /* !lint */
 
 /* system includes */
@@ -98,7 +98,8 @@ dkim_strdup(DKIM *dkim, const unsigned char *str, size_t len)
 	assert(str != NULL);
 
 	if (len == 0)
-		len = strlen(str);
+		len = strlen((char *) str);
+
 	new = dkim_malloc(dkim->dkim_libhandle, dkim->dkim_closure, len + 1);
 	if (new != NULL)
 	{
