@@ -7,7 +7,7 @@
 #define _DKIM_H_
 
 #ifndef lint
-static char dkim_h_id[] = "@(#)$Id: dkim.h,v 1.185.4.2 2008/08/28 22:14:05 msk Exp $";
+static char dkim_h_id[] = "@(#)$Id: dkim.h,v 1.197 2009/01/13 08:22:18 msk Exp $";
 #endif /* !lint */
 
 /* system includes */
@@ -26,7 +26,7 @@ static char dkim_h_id[] = "@(#)$Id: dkim.h,v 1.185.4.2 2008/08/28 22:14:05 msk E
 **  	pp == patch number
 */
 
-#define	DKIM_LIB_VERSION	0x02070200
+#define	DKIM_LIB_VERSION	0x02080100
 
 /* the basics */
 #ifndef NULL
@@ -58,6 +58,8 @@ static char dkim_h_id[] = "@(#)$Id: dkim.h,v 1.185.4.2 2008/08/28 22:14:05 msk E
 
 /* limits, macros, etc. */
 #define	BUFRSZ			1024	/* base temp buffer size */
+#define	BASE64SIZE(x)		(((x + 2) / 3) * 4)
+					/* base64 encoding growth ratio */
 #define MAXADDRESS		256	/* biggest user@host we accept */
 #define	MAXBUFRSZ		65536	/* max temp buffer size */
 #define MAXCNAMEDEPTH		3	/* max. CNAME recursion we allow */
@@ -377,9 +379,26 @@ typedef int dkim_opts_t;
 #define DKIM_LIBFLAGS_EOHCHECK		0x040
 #define DKIM_LIBFLAGS_ACCEPTV05		0x080
 #define DKIM_LIBFLAGS_FIXCRLF		0x100
+#define DKIM_LIBFLAGS_ACCEPTDK		0x200
 
 #define	DKIM_LIBFLAGS_DEFAULT		DKIM_LIBFLAGS_NONE
 
+#ifdef _FFR_DKIM_REPUTATION
+# define DKIM_REP_DEFREJECT	1001
+# define DKIM_REP_ROOT		"al.dkim-reputation.org"
+#endif /* _FFR_DKIM_REPUTATION */
+
+#ifdef USE_UNBOUND
+/*
+**  DKIM_DNSSEC -- results of DNSSEC queries
+*/
+
+# define DKIM_DNSSEC_UNKNOWN	(-1)
+# define DKIM_DNSSEC_BOGUS	0
+# define DKIM_DNSSEC_INSECURE	1
+# define DKIM_DNSSEC_SECURE	2
+#endif /* USE_UNBOUND */
+
 /*
 **  DKIM_LIB -- library handle
 */
@@ -482,6 +501,21 @@ struct dkim_hdrdiff
 **  PROTOTYPES
 */
 
+#ifdef USE_UNBOUND
+/*
+**  DKIM_SET_TRUST_ANCHOR -- set trust anchor file path
+**
+**  Parameters:
+**  	lib -- DKIM_LIB handle to update
+**  	tafile -- trust anchor file path
+** 
+**  Return value:
+**  	A DKIM_STAT constant.
+*/
+
+extern DKIM_STAT dkim_set_trust_anchor __P((DKIM_LIB *lib, char *tafile));
+#endif /* USE_UNBOUND */
+
 /*
 **  DKIM_INIT -- initialize the DKIM package
 **
@@ -675,6 +709,7 @@ extern DKIM_STAT dkim_policy_syntax __P((DKIM *dkim, u_char *str, size_t len));
 
 extern DKIM_STAT dkim_sig_syntax __P((DKIM *dkim, u_char *str, size_t len));
 
+#ifdef QUERY_CACHE
 /*
 **  DKIM_GETCACHESTATS -- retrieve cache statistics
 **
@@ -707,6 +742,7 @@ extern void dkim_getcachestats __P((u_int *queries, u_int *hits,
 */
 
 extern int dkim_flush_cache __P((DKIM_LIB *lib));
+#endif /* QUERY_CACHE */
 
 /*
 **  DKIM_MINBODY -- return number of bytes still expected
@@ -780,20 +816,37 @@ extern DKIM_STAT dkim_getsighdr __P((DKIM *dkim, u_char *buf, size_t len,
 
 extern bool dkim_sig_hdrsigned __P((DKIM_SIGINFO *sig, char *hdr));
 
+#ifdef USE_UNBOUND
+/*
+**  DKIM_SIG_GETDNSSEC -- retrieve DNSSEC results for a signature
+**
+**  Parameters:
+**  	sig -- DKIM_SIGINFO handle
+**
+**  Return value:
+**  	A DKIM_DNSSEC_* constant.
+*/
+
+extern u_int dkim_sig_getdnssec __P((DKIM_SIGINFO *sig));
+#endif /* USE_UNBOUND */
+
 /*
 **  DKIM_SIG_GETREPORTINFO -- retrieve reporting information from a key
 **
 **  Parameters:
 **  	dkim -- DKIM handle
 **  	sig -- DKIM_SIGINFO handle
-**  	hfd -- canonicalized header descriptor (returned)
-**  	bfd -- canonicalized body descriptor (returned)
-**  	addr -- address buffer
+**  	hfd -- canonicalized header descriptor (or NULL) (returned)
+**  	bfd -- canonicalized body descriptor (or NULL) (returned)
+**  	addr -- address buffer (or NULL)
 **  	addrlen -- size of addr
 **  	fmt -- format buffer (or NULL)
 **  	fmtlen -- size of fmt
 **  	opts -- options buffer (or NULL)
 **  	optslen -- size of opts
+**  	smtp -- SMTP prefix buffer (or NULL)
+**  	smtplen -- size of smtp
+**  	interval -- requested report interval (or NULL)
 **
 **  Return value:
 **  	A DKIM_STAT_* constant.
@@ -803,7 +856,9 @@ extern DKIM_STAT dkim_sig_getreportinfo __P((DKIM *dkim, DKIM_SIGINFO *sig,
                                              int *hfd, int *bfd,
                                              u_char *addr, size_t addrlen,
                                              u_char *fmt, size_t fmtlen,
-                                             u_char *opts, size_t optslen));
+                                             u_char *opts, size_t optslen,
+                                             u_char *smtp, size_t smtplen,
+                                             u_int *interval));
 
 /*
 **  DKIM_SIG_GETIDENTITY -- retrieve identity of the signer
@@ -1242,17 +1297,34 @@ extern void dkim_sig_ignore __P((DKIM_SIGINFO *siginfo));
 extern DKIM_STAT dkim_policy __P((DKIM *dkim, dkim_policy_t *pcode,
                                   DKIM_PSTATE *pstate));
 
+#ifdef USE_UNBOUND
+/*
+**  DKIM_POLICY_GETDNSSEC -- retrieve DNSSEC results for a policy
+**
+**  Parameters:
+**  	dkim -- DKIM handle
+**
+**  Return value:
+**  	A DKIM_DNSSEC_* constant.
+*/
+
+extern u_int dkim_policy_getdnssec __P((DKIM *dkim));
+#endif /* USE_UNBOUND */
+
 /*
 **  DKIM_POLICY_GETREPORTINFO -- retrieve reporting information from policy
 **
 **  Parameters:
 **  	dkim -- DKIM handle
-**  	addr -- address buffer
+**  	addr -- address buffer (or NULL)
 **  	addrlen -- size of addr
 **  	fmt -- format buffer (or NULL)
 **  	fmtlen -- size of fmt
 **  	opts -- options buffer (or NULL)
 **  	optslen -- size of opts
+**  	smtp -- SMTP prefix buffer (or NULL)
+**  	smtplen -- size of smtp
+**  	interval -- requested report interval (or NULL)
 **
 **  Return value:
 **  	A DKIM_STAT_* constant.
@@ -1261,7 +1333,9 @@ extern DKIM_STAT dkim_policy __P((DKIM *dkim, dkim_policy_t *pcode,
 extern DKIM_STAT dkim_policy_getreportinfo __P((DKIM *dkim,
                                                 u_char *addr, size_t addrlen,
                                                 u_char *fmt, size_t fmtlen,
-                                                u_char *opts, size_t optslen));
+                                                u_char *opts, size_t optslen,
+                                                u_char *smtp, size_t smtplen,
+                                                u_int *interval));
 
 /*
 **  DKIM_SIG_PROCESS -- process a signature
@@ -1434,6 +1508,28 @@ extern DKIM_STAT dkim_setpartial __P((DKIM *dkim, bool value));
 
 extern DKIM_STAT dkim_set_margin __P((DKIM *dkim, int value));
 
+#ifdef _FFR_DKIM_REPUTATION
+/*
+**  DKIM_GET_REPUTATION -- query reputation service about a signature
+**
+**  Parameters:
+**  	dkim -- DKIM handle
+**  	sig -- DKIM_SIGINFO handle
+**  	qroot -- query root
+**  	rep -- integer reputation (returned)
+**
+**  Return value:
+**  	DKIM_STAT_OK -- "rep" now contains a reputation
+**  	DKIM_STAT_NOKEY -- no reputation data available
+**  	DKIM_STAT_CANTVRFY -- data retrieval error of some kind
+**  	DKIM_STAT_INTERNAL -- internal error of some kind
+*/
+
+extern DKIM_STAT
+dkim_get_reputation __P((DKIM *dkim, DKIM_SIGINFO *sig, char *qroot,
+                         int *rep));
+#endif /* _FFR_DKIM_REPUTATION */
+
 /*
 **  RFC2822_MAILBOX_SPLIT -- extract the userid and host from a structured
 **                           header
