@@ -4,7 +4,7 @@
 */
 
 #ifndef lint
-static char util_c_id[] = "@(#)$Id: util.c,v 1.24 2008/05/08 00:19:33 msk Exp $";
+static char util_c_id[] = "@(#)$Id: util.c,v 1.26 2008/10/31 03:06:05 msk Exp $";
 #endif /* !lint */
 
 /* system includes */
@@ -139,9 +139,9 @@ dkim_hdrlist(u_char *buf, size_t buflen, u_char **hdrlist, bool first)
 {
 	int c;
 	int len;
-	char *p;
-	char *q;
-	char *end;
+	u_char *p;
+	u_char *q;
+	u_char *end;
 
 	assert(buf != NULL);
 	assert(hdrlist != NULL);
@@ -153,13 +153,13 @@ dkim_hdrlist(u_char *buf, size_t buflen, u_char **hdrlist, bool first)
 
 		if (!first)
 		{
-			len = sm_strlcat(buf, "|", buflen);
+			len = sm_strlcat((char *) buf, "|", buflen);
 			if (len >= buflen)
 				return FALSE;
 		}
 		else
 		{
-			len = strlen(buf);
+			len = strlen((char *) buf);
 		}
 
 		first = FALSE;
@@ -305,7 +305,7 @@ dkim_qp_decode(unsigned char *in, unsigned char *out, int outlen)
 	unsigned char const *start;
 	unsigned char const *stop;
 	unsigned char *end;
-	unsigned char const *hexdigits = "0123456789ABCDEF";
+	char const *hexdigits = "0123456789ABCDEF";
 
 	assert(in != NULL);
 	assert(out != NULL);
@@ -669,8 +669,8 @@ dkim_check_dns_reply(unsigned char *ansbuf, size_t anslen,
 	     qdcount--)
 	{
 		/* copy it first */
-		(void) dn_expand((unsigned char *) ansbuf, eom, cp, name,
-		                 sizeof name);
+		(void) dn_expand((unsigned char *) ansbuf, eom, cp,
+		                 (RES_UNC_T) name, sizeof name);
 
 		if ((n = dn_skipname(cp, eom)) < 0)
 			return 2;
@@ -727,7 +727,7 @@ dkim_check_dns_reply(unsigned char *ansbuf, size_t anslen,
 		if (type == T_CNAME)
 		{
 			if ((n = dn_expand((u_char *) ansbuf, eom, cp,
-			                   name, sizeof name)) < 0)
+			                   (RES_UNC_T) name, sizeof name)) < 0)
 				return 2;
 
 			cp += n;
@@ -762,3 +762,82 @@ dkim_check_dns_reply(unsigned char *ansbuf, size_t anslen,
 
 	return (trunc ? 1 : 0);
 }
+
+#ifdef USE_UNBOUND
+/*
+**  DKIM_TIMESPEC_PAST -- return TRUE iff the time described by a timespec
+**                        structure has passed
+**
+**  Parameters:
+**  	ts -- timespec structure to evaluate
+**
+**  Return value:
+**  	TRUE if "tv" refers to a time in the past, false otherwise.
+*/
+
+bool
+dkim_timespec_past(struct timespec *ts)
+{
+	struct timeval now;
+
+	assert(ts != NULL);
+
+	(void) gettimeofday(&now, NULL);
+
+	if (now.tv_sec > ts->tv_sec ||
+	    (now.tv_sec == ts->tv_sec && now.tv_usec * 1000 > ts->tv_nsec))
+		return TRUE;
+	else
+		return FALSE;
+}
+
+/*
+**  DKIM_WAIT_FD -- wait for a descriptor to become read-ready
+**
+**  Parameters:
+**  	fd -- descriptor of interest
+**  	until -- maximum wait time
+**
+**  Return value:
+**  	1 -- descriptor is ready
+**  	0 -- timeout
+**  	-1 -- error
+*/
+
+int
+dkim_wait_fd(int fd, struct timespec *until)
+{
+	fd_set fds;
+	struct timeval now;
+	struct timeval left;
+
+	assert(fd >= 0);
+	assert(until != NULL);
+
+	(void) gettimeofday(&now, NULL);
+
+	if (until->tv_sec < now.tv_sec ||
+	    (until->tv_sec == now.tv_sec &&
+	     until->tv_nsec < now.tv_usec * 1000))
+	{
+		left.tv_sec = 0;
+		left.tv_usec = 0;
+	}
+	else
+	{
+		left.tv_sec = until->tv_sec - now.tv_sec;
+		left.tv_usec = until->tv_nsec / 1000 - now.tv_usec;
+
+		if (until->tv_nsec / 1000 < now.tv_usec)
+		{
+			left.tv_usec += 1000000;
+			left.tv_sec--;
+		}
+	}
+
+	FD_ZERO(&fds);
+	FD_SET(fd, &fds);
+
+	return select(fd + 1, &fds, NULL, NULL, &left);
+}
+#endif /* USE_UNBOUND */
